/*
 *  KnotTTY is a remote secure shell using the new public key exchange protocol
 *  with the Braid Group Cryptosystems
 *
 *  Copyright (C) 2006 Adrien Pinet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 */

#include "CKnotttyParam.h"

#include <termios.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

CKnotttyParam::CKnotttyParam()
{
	hostName[0] = 0;
	userName[0] = 0;
	password[0] = 0;
	srcFilePathName[0] = 0;
	destFilePathName[0] = 0;
	port = 1789;
}

CKnotttyParam::~CKnotttyParam()
{
	Destroy();
}

CKnotttyParam::SESSIONMOD CKnotttyParam::Init(int argc, char* argv[])
{
	Destroy();

	argc--;
	argv++;

	if(argc == 1)
	{
		if(!SearchPort(argv[0], &port))
		port = 1789;

		if(SearchUserName(argv[0], userName) && SearchHostName(argv[0], hostName))
		return SHELL;

		return UNKNOW;
	}

	if(argc == 2)
	{
		if(strcmp(argv[0], "-X") == 0)
		{
			if(!SearchPort(argv[1], &port))
			port = 1789;

			if(SearchUserName(argv[1], userName) && SearchHostName(argv[1], hostName))
			return XSHELL;

			return UNKNOW;
		}

		if(SearchCharPos(argv[0], '@') > 0)
		{
			if(!SearchPort(argv[0], &port))
			port = 1789;

			if(SearchUserName(argv[0], userName) && SearchHostName(argv[0], hostName) && SearchPathName(argv[0], srcFilePathName)&& SearchPathName(argv[1], destFilePathName))
			return FTPDOWNLOAD;

			return UNKNOW;
		}

		if(SearchCharPos(argv[1], '@') > 0)
		{
			if(!SearchPort(argv[1], &port))
			port = 1789;

			if(SearchPathName(argv[0], srcFilePathName) && SearchUserName(argv[1], userName) && SearchHostName(argv[1], hostName) && SearchPathName(argv[1], destFilePathName))
			return FTPUPLOAD;

			return UNKNOW;
		}

		return UNKNOW;
	}

	return UNKNOW;
}



char* CKnotttyParam::GetPassword()
{
	termios stty;

	tcgetattr(0, &stty);
	stty.c_cc[VTIME] = 600 * 10;
	stty.c_cc[VMIN] = 1;

	printf("Password:\n");

	stty.c_lflag &= ~(ICANON | ECHO);
	tcsetattr(0,TCSANOW, &stty);

	fgets(password, 100, stdin);
	password[strlen(password) - 1] = 0;

	stty.c_lflag |= (ICANON | ECHO);
	tcsetattr(0,TCSANOW, &stty);

	return password;
}

int CKnotttyParam::SearchUserName(char param[], char userName[])
{
	int i;
	int atPos = SearchCharPos(param, '@');

	if(atPos == -1)
	return 0;

	for(i = 0 ; i != atPos ; i++)
	userName[i] = param[i];

	userName[i] = 0;

	return 1;
}

int CKnotttyParam::SearchHostName(char param[], char hostName[])
{
	int i;
	int atPos = SearchCharPos(param, '@');
	int slashPos = SearchCharPos(param, '/');
	int dpointPos = SearchCharPos(param, ':');
	int zeroPos = SearchCharPos(param, '\0');

	if(atPos == -1)
	return 0;

	for(i = atPos + 1 ; i != slashPos && i != dpointPos && i != zeroPos ; i++)
	hostName[i - (atPos + 1)] = param[i];

	hostName[i - (atPos + 1)] = 0;

	return 1;
}

int CKnotttyParam::SearchPathName(char param[], char pathName[])
{
	int i;
	int atPos = SearchCharPos(param, '@');
	int dpointPos = SearchCharPos(param, ':');
	int zeroPos = SearchCharPos(param, '\0');

	if(dpointPos == -1 && atPos == -1)
	{
		strcpy(pathName, param);
		return 1;
	}

	if(dpointPos == -1 && atPos)
	return 0;

	for(i = dpointPos + 1 ; i != zeroPos ; i++)
	pathName[i - (dpointPos + 1)] = param[i];

	pathName[i - (dpointPos + 1)] = 0;

	return 1;
}

int CKnotttyParam::SearchPort(char param[], int* pPort)
{
	char sPort[100];

	int i;
	int slashPos = SearchCharPos(param, '/');
	int dpointPos = SearchCharPos(param, ':');
	int zeroPos = SearchCharPos(param, '\0');

	if(slashPos == -1)
	return 0;

	for(i = slashPos + 1 ; i!= dpointPos && i != zeroPos ; i++)
	sPort[i - (slashPos + 1)] = param[i];

	sPort[i - (slashPos + 1)] = 0;

	if(atoi(sPort) == 0)
	return 0;

	*pPort = atoi(sPort);

	return 1;
}

void CKnotttyParam::PrintHelp()
{
	printf("\n<--------------------  KnotTTY Help  -------------------->\n");
	printf(">Remote shell :                                              \n");
	printf("  knottty username@hostname[/port]                          \n");
	printf(">File Transfert :                                          \n");
	printf("  knottty username@hostname[/port]:srcfilepath destfilepath \n");
	printf("  knottty srcfilepath username@hostname[/port]:destfilepath \n");
	printf("\n");
}

int CKnotttyParam::SearchCharPos(char param[], char car)
{
	int size = strlen(param);

	for(int i = 0 ; i <= size ; i++)
	{
		if(param[i] == car)
		return i;
	}

	return -1;
}

void CKnotttyParam::Destroy()
{
	for(int i = 0 ; i < 100 ; i++)
	{
		userName[i] = 0xFF;
		password[i] = 0xFF;
		userName[i] = 0x00;
		password[i] = 0x00;
	}
	
	for(int i = 0 ; i < 200 ; i++)
	{
		hostName[i] = 0xFF;
		srcFilePathName[i] = 0xFF;
		destFilePathName[i] = 0xFF;
		hostName[i] = 0x00;
		srcFilePathName[i] = 0x00;
		destFilePathName[i] = 0x00;
	}

	port = 1789;
}
