/*
 *  KnotTTY is a remote secure shell using the new public key exchange protocol
 *  with the Braid Group Cryptosystems
 *
 *  Copyright (C) 2006 Adrien Pinet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 */

#include "CSessionShell.h"

#include "CBuffer.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <pthread.h>
#include <unistd.h> // sleep
#include <signal.h>

CSessionShell::CSessionShell()
{
	termios stty;
	tcgetattr(0, &stty);

	stty.c_lflag &= ~(ICANON | ECHO);
	stty.c_cc[VTIME] = 600 * 10;
	stty.c_cc[VMIN] = 1;

	tcsetattr(0,TCSANOW, &stty);

	signal(SIGINT, InterceptSignal);
	threadIOTermRunning = 0;
}

CSessionShell::~CSessionShell()
{
	termios stty;
	tcgetattr(0, &stty);

	stty.c_lflag |= (ICANON | ECHO);
	stty.c_cc[VTIME] = 600 * 10;
	stty.c_cc[VMIN] = 1;

	tcsetattr(0,TCSANOW, &stty);

	Stop();
}

int CSessionShell::Start(char ip[], int port, char userName[], char password[])
{
	CBuffer SessionMod(1);
	CBuffer Status(1);

	SessionMod.GetBuffer()[0] = SHELL;

	if(!SecureConnection.ConnectToServer(ip, port))
	return 0;

	if(!SecureConnection.SecureConnection())
	return 0;
	
	if(!SecureConnection.SecureSend((uByte*)userName, strlen(userName) + 1))
	return 0;

	if(!SecureConnection.SecureSend((uByte*)password, strlen(password) + 1))
	return 0;

	if(!SecureConnection.SecureSend(SessionMod.GetBuffer(), SessionMod.GetMaxBufferSize()))
	return 0;

	if(!SecureConnection.SecureReceive(Status.GetBuffer(), Status.GetMaxBufferSize(), &Status.bufferSize))
	return 0;

	if(Status.GetBuffer()[0] == NOAUTHENTICATED)
	return 0;

	StartThreadIOTerm();

	pthread_join(threadITerm, NULL);
	pthread_join(threadOTerm, NULL);

	return 1;
}

int CSessionShell::Stop()
{
	if(threadIOTermRunning)
	{
		pthread_cancel(threadITerm);
		pthread_cancel(threadOTerm);
	}

	threadIOTermRunning = 0;

	SecureConnection.Close();

	return 1;
}

int CSessionShell::StartThreadIOTerm()
{
	pthread_attr_init(&thread_attrITerm);
	pthread_attr_init(&thread_attrOTerm);

	if(pthread_create(&threadITerm, &thread_attrITerm, CSessionShell::ThreadITerm, this))
	return 0;

	if(pthread_create(&threadOTerm, &thread_attrOTerm, CSessionShell::ThreadOTerm, this))
	return 0;

	threadIOTermRunning = 1;

	return 1;
}

void* CSessionShell::ThreadITerm(void* pvSessionShell)
{
	CSessionShell* pSessionShell = (CSessionShell*) pvSessionShell;

	CBuffer Data(CSecureConnection::MAXSOCKETDATASIZE);

	while(pSessionShell->SecureConnection.SecureReceive(Data.GetBuffer(), Data.GetMaxBufferSize(), &Data.bufferSize))
	{
		for(uInt32 i = 0 ; i < Data.GetBufferSize() ; i++)
		printf("%c", Data.GetBuffer(i));
		fflush(stdout);
	}

	pthread_cancel(pSessionShell->threadOTerm);
	return NULL;
}

void* CSessionShell::ThreadOTerm(void* pvSessionShell)
{
	CSessionShell* pSessionShell = (CSessionShell*) pvSessionShell;

	CBuffer Char(1);
	
	while(read(0, Char.GetBuffer(), Char.GetMaxBufferSize()) >= 0)
	{
		if(!pSessionShell->SecureConnection.SecureSend(Char.GetBuffer(), Char.GetMaxBufferSize()))
		{
			pthread_cancel(pSessionShell->threadITerm);
			return NULL;
		}
	}

	pthread_cancel(pSessionShell->threadITerm);
	return NULL;
}

void CSessionShell::InterceptSignal(int signal)
{
	if(signal == SIGINT)
	{
		termios stty;
		tcgetattr(0, &stty);
	
		stty.c_lflag |= (ICANON | ECHO);
		stty.c_cc[VTIME] = 600 * 10;
		stty.c_cc[VMIN] = 1;
	
		tcsetattr(0,TCSANOW, &stty);
	
		exit(0);
	}
}
