/*
 *  KnotTTY is a remote secure shell using the new public key exchange protocol
 *  with the Braid Group Cryptosystems
 *
 *  Copyright (C) 2006 Adrien Pinet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 */

#ifndef __CSESSIONSHELL_H__
#define __CSESSIONSHELL_H__

#include "CSecureConnection.h"

#include <pthread.h>
#include <termios.h>

class CSessionShell
{
public:
	enum STATUS
	{
		AUTHENTICATED = 0x00,
		NOAUTHENTICATED = 0x01
	};

	enum SESSIONMOD
	{
		SHELL = 0x00,
		XSHELL = 0x01,
		FTP = 0x02
	};

public:
	CSessionShell();
	~CSessionShell();

	int Start(char ip[], int port, char userName[], char password[]);
	int Stop();

private:
	int StartThreadIOTerm();
	int threadIOTermRunning;
	static void* ThreadITerm(void* pvSessionShell);
	static void* ThreadOTerm(void* pvSessionShell);

	static void InterceptSignal(int signal);

private:
	CSecureConnection SecureConnection;
	
	pthread_t threadITerm;
	pthread_t threadOTerm;
	
	pthread_attr_t thread_attrITerm;
	pthread_attr_t thread_attrOTerm;

	termios sttyOld, sttyNew;
};

#endif // __CSESSIONSHELL_H__
