/*
 *  KnotTTY is a remote secure shell using the new public key exchange protocol
 *  with the Braid Group Cryptosystems
 *
 *  Copyright (C) 2006 Adrien Pinet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 */

#include "CSessionXShell.h"

#include "CBuffer.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <pthread.h>
#include <unistd.h>
#include <signal.h>

CSessionXShell::CSessionXShell()
{
	termios stty;
	tcgetattr(0, &stty);

	stty.c_lflag &= ~(ICANON | ECHO);
	stty.c_cc[VTIME] = 600 * 10;
	stty.c_cc[VMIN] = 1;

	tcsetattr(0,TCSANOW, &stty);

	signal(SIGINT, InterceptSignal);

	threadIOTermRunning = 0;
	for(uInt32 i = 0 ; i < 255 ; i++)
	xConnectionList[i].pXConnection = NULL;
}

CSessionXShell::~CSessionXShell()
{
	termios stty;
	tcgetattr(0, &stty);

	stty.c_lflag |= (ICANON | ECHO);
	stty.c_cc[VTIME] = 600 * 10;
	stty.c_cc[VMIN] = 1;

	tcsetattr(0,TCSANOW, &stty);

	Stop();
}

int CSessionXShell::Start(char ip[], int port, char userName[], char password[])
{
	CBuffer SessionMod(1);
	CBuffer Status(1);

	SessionMod.GetBuffer()[0] = XSHELL;

	if(!SecureConnection.ConnectToServer(ip, port))
	return 0;

	if(!SecureConnection.SecureConnection())
	return 0;
	
	if(!SecureConnection.SecureSend((uByte*)userName, strlen(userName) + 1))
	return 0;

	if(!SecureConnection.SecureSend((uByte*)password, strlen(password) + 1))
	return 0;

	if(!SecureConnection.SecureSend(SessionMod.GetBuffer(), SessionMod.GetMaxBufferSize()))
	return 0;

	if(!SecureConnection.SecureReceive(Status.GetBuffer(), Status.GetMaxBufferSize(), &Status.bufferSize))
	return 0;

	if(Status.GetBuffer()[0] == NOAUTHENTICATED)
	return 0;

	if(!ChannelManager.StartChannelManager(&SecureConnection, this, 255))
	return 0;

	sleep(4); // Waiting opening shell channel event

	if(!threadIOTermRunning)
	return 0;

	pthread_join(threadITerm, NULL);
	pthread_join(threadOTerm, NULL);

	return 1;
}

int CSessionXShell::Stop()
{
	if(threadIOTermRunning)
	{
		pthread_cancel(threadITerm);
		pthread_cancel(threadOTerm);
	}

	threadIOTermRunning = 0;

	SecureConnection.Close();

	for(uInt32 i = 0 ; i < 255 ; i++)
	{
		if(xConnectionList[i].pXConnection)
		{
			pthread_cancel(xConnectionList[i].threadIXConnection);
			pthread_cancel(xConnectionList[i].threadOXConnection);
			delete xConnectionList[i].pXConnection;
			xConnectionList[i].pXConnection = NULL;
		}
	}

	return 1;
}

int CSessionXShell::StartThreadIOTerm()
{
	pthread_attr_init(&thread_attrITerm);
	pthread_attr_init(&thread_attrOTerm);

	if(pthread_create(&threadITerm, &thread_attrITerm, CSessionXShell::ThreadITerm, this))
	return 0;

	if(pthread_create(&threadOTerm, &thread_attrOTerm, CSessionXShell::ThreadOTerm, this))
	return 0;

	threadIOTermRunning = 1;

	return 1;
}

void* CSessionXShell::ThreadITerm(void* pvSessionXShell)
{
	CSessionXShell* pSessionXShell = (CSessionXShell*) pvSessionXShell;

	CBuffer Data(CSecureConnection::MAXSOCKETDATASIZE - 2);

	while(pSessionXShell->ChannelManager.Read(0, Data.GetBuffer(), Data.GetMaxBufferSize(), &Data.bufferSize))
	{
		for(uInt32 i = 0 ; i < Data.GetBufferSize() ; i++)
		printf("%c", Data.GetBuffer(i));
		fflush(stdout);
	}

	pthread_cancel(pSessionXShell->threadOTerm);
	return NULL;
}

void* CSessionXShell::ThreadOTerm(void* pvSessionXShell)
{
	CSessionXShell* pSessionXShell = (CSessionXShell*) pvSessionXShell;

	CBuffer Char(1);
	
	while(read(0, Char.GetBuffer(), Char.GetMaxBufferSize()) >= 0)
	{
		if(!pSessionXShell->ChannelManager.Write(0, Char.GetBuffer(), Char.GetMaxBufferSize()))
		{
			pthread_cancel(pSessionXShell->threadITerm);
			return NULL;
		}
	}

	pthread_cancel(pSessionXShell->threadITerm);
	return NULL;
}


int CSessionXShell::StartThreadIOXConnection(uInt32 idChannelXConnection)
{
	CXConnection* pXConnection = new CXConnection;

	if(!pXConnection->ConnectToServer("localhost", 6000))
	{
		delete pXConnection;
		return 0;
	}
	
	xConnectionList[idChannelXConnection].idChannelXConnection = idChannelXConnection;
	xConnectionList[idChannelXConnection].pChannelManager = &ChannelManager;
	xConnectionList[idChannelXConnection].pXConnection = pXConnection;

	pthread_attr_init(&xConnectionList[idChannelXConnection].thread_attrIXConnection);
	pthread_attr_init(&xConnectionList[idChannelXConnection].thread_attrOXConnection);

	if(pthread_create(&xConnectionList[idChannelXConnection].threadIXConnection, &xConnectionList[idChannelXConnection].thread_attrIXConnection, CSessionXShell::ThreadIXConnection, &xConnectionList[idChannelXConnection]))
	return 0;

	if(pthread_create(&xConnectionList[idChannelXConnection].threadOXConnection, &xConnectionList[idChannelXConnection].thread_attrOXConnection, CSessionXShell::ThreadOXConnection, &xConnectionList[idChannelXConnection]))
	return 0;

	return 1;
}

void* CSessionXShell::ThreadIXConnection(void* pvThreadXConnectionData)
{
	SThreadXConnectionData* pThreadXConnectionData = (SThreadXConnectionData*) pvThreadXConnectionData;

	uInt32 idChannelXConnection = pThreadXConnectionData->idChannelXConnection;
	CXConnection* pXConnection = pThreadXConnectionData->pXConnection;
	CChannelManager<CSecureConnection, CSessionXShell>* pChannelManager = pThreadXConnectionData->pChannelManager;

	CBuffer Data(CSecureConnection::MAXSOCKETDATASIZE - 2);

	while(pChannelManager->Read(idChannelXConnection, Data.GetBuffer(), Data.GetMaxBufferSize(), &Data.bufferSize))
	{
		if(!pXConnection->Send(Data.GetBuffer(), Data.GetBufferSize()))
		{
			pthread_cancel(pThreadXConnectionData->threadOXConnection);
			delete pThreadXConnectionData->pXConnection;
			pThreadXConnectionData->pXConnection = NULL;
			pChannelManager->Close(idChannelXConnection);
			return NULL;
		}
	}

	pthread_cancel(pThreadXConnectionData->threadOXConnection);
	delete pThreadXConnectionData->pXConnection;
	pThreadXConnectionData->pXConnection = NULL;
	pChannelManager->Close(idChannelXConnection);
	return NULL;
}

void* CSessionXShell::ThreadOXConnection(void* pvThreadXConnectionData)
{
	SThreadXConnectionData* pThreadXConnectionData = (SThreadXConnectionData*) pvThreadXConnectionData;

	uInt32 idChannelXConnection = pThreadXConnectionData->idChannelXConnection;
	CXConnection* pXConnection = pThreadXConnectionData->pXConnection;
	CChannelManager<CSecureConnection, CSessionXShell>* pChannelManager = pThreadXConnectionData->pChannelManager;

	CBuffer Data(CSecureConnection::MAXSOCKETDATASIZE - 2);

	while(pXConnection->Receive(Data.GetBuffer(), Data.GetMaxBufferSize(), &Data.bufferSize))
	{
		if(!pChannelManager->Write(idChannelXConnection, Data.GetBuffer(), Data.GetBufferSize()))
		{
			pthread_cancel(pThreadXConnectionData->threadIXConnection);
			delete pThreadXConnectionData->pXConnection;
			pThreadXConnectionData->pXConnection = NULL;
			pChannelManager->Close(idChannelXConnection);
			return NULL;
		}
	}

	pthread_cancel(pThreadXConnectionData->threadIXConnection);
	delete pThreadXConnectionData->pXConnection;
	pThreadXConnectionData->pXConnection = NULL;
	pChannelManager->Close(idChannelXConnection);
	return NULL;
}

void CSessionXShell::OnOpen(uInt32 idChannel)
{
	if(idChannel == 0)
	StartThreadIOTerm();
	else
	StartThreadIOXConnection(idChannel);
}

void CSessionXShell::OnClose(uInt32 idChannel)
{
	if(idChannel == 0)
	{
		Stop();
		return;
	}

	if(xConnectionList[idChannel].pXConnection)
	{
		pthread_cancel(xConnectionList[idChannel].threadIXConnection);
		pthread_cancel(xConnectionList[idChannel].threadOXConnection);

		delete xConnectionList[idChannel].pXConnection;
		xConnectionList[idChannel].pXConnection = NULL;
	}
}

void CSessionXShell::InterceptSignal(int signal)
{
	if(signal == SIGINT)
	{
		termios stty;
		tcgetattr(0, &stty);
	
		stty.c_lflag |= (ICANON | ECHO);
		stty.c_cc[VTIME] = 600 * 10;
		stty.c_cc[VMIN] = 1;
	
		tcsetattr(0,TCSANOW, &stty);
	
		exit(0);
	}
}
