/*
 *  KnotTTY is a remote secure shell using the new public key exchange protocol
 *  with the Braid Group Cryptosystems
 *
 *  Copyright (C) 2006 Adrien Pinet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 */

#include "CXConnection.h"

#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <netdb.h>
#include <string.h>
#include <stdio.h>
#include <signal.h>


CXConnection::CXConnection()
{
}

CXConnection::~CXConnection()
{
	Close();
}

int CXConnection::ConnectToServer(char adressServer[], int portServer)
{
	signal(SIGPIPE, SIG_IGN);
	
	pthread_mutex_init(&mutexToSend, NULL);
	pthread_mutex_init(&mutexToReceive, NULL);

	sockaddr_in address;
        hostent *pHost;

	if ((pHost = gethostbyname(adressServer)) == NULL)
	return 0;

	memset(&address, 0x00, sizeof(sockaddr_in));
	memcpy(&address.sin_addr, pHost->h_addr, pHost->h_length);

	address.sin_family = pHost->h_addrtype;
	address.sin_port = htons(portServer);

	if((socketServer = socket(pHost->h_addrtype, SOCK_STREAM, 0)) < 0)
	return 0;

	if(connect(socketServer, (sockaddr*)&address, sizeof(sockaddr_in)) < 0)
	return 0;

	return 1;
}

void CXConnection::Close()
{
	shutdown(socketServer, SHUT_RDWR);
	close(socketServer);

	pthread_mutex_unlock(&mutexToSend);
	pthread_mutex_unlock(&mutexToReceive);

	pthread_mutex_destroy(&mutexToSend);
	pthread_mutex_destroy(&mutexToReceive);
}


int CXConnection::Send(uByte buffer[], uInt32 bufferSize)
{
	LockToSend();

	if(bufferSize > MAXSOCKETDATASIZE)
	{
		UnLockToSend();
		return 0;
	}

	uInt32 bufferSizeSend = 0;

	while(bufferSizeSend < bufferSize)
	{
		int currentBufferSizeSend = write(socketServer, buffer + bufferSizeSend, bufferSize - bufferSizeSend);
		
		if(currentBufferSizeSend <= 0)
		{
			UnLockToSend();
			return 0;
		}

		bufferSizeSend += currentBufferSizeSend;
	}

	UnLockToSend();
	return 1;
}

int CXConnection::Receive(uByte data[], uInt32 maxDataSize, uInt32* pDataSize)
{
	LockToReceive();

	if(maxDataSize > MAXSOCKETDATASIZE)
	maxDataSize = MAXSOCKETDATASIZE;

	int currentDataSizeReceive = read(socketServer, data, maxDataSize);

	if(currentDataSizeReceive <= 0)
	{
		UnLockToReceive();
		return 0;
	}

	*pDataSize = currentDataSizeReceive;

	UnLockToReceive();
	return 1;
}

void CXConnection::LockToSend()
{
	pthread_mutex_lock(&mutexToSend);
}

void CXConnection::UnLockToSend()
{
	pthread_mutex_unlock(&mutexToSend);
}

void CXConnection::LockToReceive()
{
	pthread_mutex_lock(&mutexToReceive);
}

void CXConnection::UnLockToReceive()
{
	pthread_mutex_unlock(&mutexToReceive);
}
