/*
 *  KnotTTY is a remote secure shell using the new public key exchange protocol
 *  with the Braid Group Cryptosystems
 *
 *  Copyright (C) 2006 Adrien Pinet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 */

#include "CSessionShell.h"
#include "CSessionXShell.h"
#include "CSessionFTP.h"
#include "CKnotttyParam.h"

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <sys/signal.h>




int main(int argc,char* argv[])
{
	CKnotttyParam KnotttyParam;

	CKnotttyParam::SESSIONMOD sessionMod = KnotttyParam.Init(argc, argv);

	if(sessionMod == CKnotttyParam::UNKNOW)
	{
		CKnotttyParam::PrintHelp();
		return 1;
	}

	if(sessionMod == CKnotttyParam::SHELL)
	{
		char* password = KnotttyParam.GetPassword();

		CSessionShell SessionShell;
		if(!SessionShell.Start(KnotttyParam.GetHostName(), KnotttyParam.GetPort(), KnotttyParam.GetUserName(), password))
		{
			printf("Connection Error\n");
			return 1;
		}
	}

	if(sessionMod == CKnotttyParam::XSHELL)
	{
		char* password = KnotttyParam.GetPassword();

		CSessionXShell SessionXShell;
		if(!SessionXShell.Start(KnotttyParam.GetHostName(), KnotttyParam.GetPort(), KnotttyParam.GetUserName(), password))
		{
			printf("Connection Error\n");
			return 1;
		}
	}

	if(sessionMod == CKnotttyParam::FTPUPLOAD || sessionMod == CKnotttyParam::FTPDOWNLOAD)
	{
		char* password = KnotttyParam.GetPassword();

		CSessionFTP SessionFTP;
		if(!SessionFTP.Init(KnotttyParam.GetHostName(), KnotttyParam.GetPort(), KnotttyParam.GetUserName(), password))
		{
			printf("Connection Error\n");
			return 1;
		}

		if(sessionMod == CKnotttyParam::FTPUPLOAD)
		{
			if(!SessionFTP.Upload(KnotttyParam.GetSrcFilePathName(), KnotttyParam.GetDestFilePathName()))
			{
				printf("\nTransfert Error\n");
				return 1;
			}
		}
		
		if(sessionMod == CKnotttyParam::FTPDOWNLOAD)
		{
			if(!SessionFTP.Download(KnotttyParam.GetSrcFilePathName(), KnotttyParam.GetDestFilePathName()))
			{
				printf("\nTransfert Error\n");
				return 1;
			}
		}

		printf("\nTransfert Successfull\n");
	}

	printf("Connection to %s Closed\n", KnotttyParam.GetHostName());
	return 0;
}
