/*
 *  KnotTTY is a remote secure shell using the new public key exchange protocol
 *  with the Braid Group Cryptosystems
 *
 *  Copyright (C) 2006 Adrien Pinet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 */

#include "CAes.h"
#include <string.h>

CAes::CAes()
{

}

CAes::CAes(char password[])
{
	Init(password);
}

CAes::CAes(uByte password[])
{
	Init(password);
}

CAes::~CAes()
{
	memset(padding, 0x00, sizeof(padding));
	memset(&context, 0X00, sizeof(context));
}

void CAes::Init(char password[])
{
	uByte expendKey[32];
	uInt32 i, passwordSize = strlen(password);

	for(i = 0 ; i < 32 ; i++)
	expendKey[i] = 0;
	
	for(i = 0 ; i < 32 || i < passwordSize ; i++)
	expendKey[i % 32] ^= password[i % passwordSize];
	
	for(i = 0 ; i < 16 ; i++)
	padding[i] = 0;
	
	for(i = 0 ; i < 32 ; i++)
	padding[i % 16] ^= expendKey[i];

	aes_set_key(&context, expendKey, 256);
}
void CAes::Init(uByte password[])
{
	uInt32 i;
	uByte expendKey[32];

	for(i = 0 ; i < 32 ; i++)
	expendKey[i] = password[i];

	for(i = 0 ; i < 16 ; i++)
	padding[i] = 0;
	
	for(i = 0 ; i < 32 ; i++)
	padding[i % 16] ^= expendKey[i];

	aes_set_key(&context, expendKey, 256);
}

void CAes::Encrypt(uByte bufferIn[], uByte bufferOut[], uInt32 bufferSize)
{
	uInt32 i, k = 0;
	uInt32 numBlock = bufferSize / 16;
	
	for(i = 0 ; i < numBlock ; i++)
	{
		aes_encrypt(&context, bufferIn + k, bufferOut + k);
		k += 16;
	}

	uInt32 restSize = bufferSize % 16;
	
	if(restSize == 0)
	return;

	ComputeNextPadding();

	for( i = 0 ; i < restSize ; i++)
	bufferOut[k + i] = padding[i] ^ bufferIn[k + i];
}


void CAes::Decrypt(uByte bufferIn[], uByte bufferOut[],uInt32 bufferSize)
{
	uInt32 i, k = 0;
	uInt32 numBlock = bufferSize / 16;
	
	for(i = 0 ; i < numBlock ; i++)
	{
		aes_decrypt(&context, bufferIn + k, bufferOut + k);
		k += 16;
	}

	uInt32 restSize = bufferSize % 16;
	
	if(restSize == 0)
	return;

	ComputeNextPadding();

	for(i = 0 ; i < restSize ; i++)
	bufferOut[k + i] = padding[i] ^ bufferIn[k + i];
}

void CAes::ComputeNextPadding(void)
{
	uByte tempPadding[16];

	for(uInt32 i = 0 ; i < 16 ; i++)
	tempPadding[i] = padding[i];
		
	aes_encrypt(&context,tempPadding,padding);
}
