/*
 *  KnotTTY is a remote secure shell using the new public key exchange protocol
 *  with the Braid Group Cryptosystems
 *
 *  Copyright (C) 2006 Adrien Pinet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 */

#include "CBraid.h"

#include "CSHADigest.h"

#include <sys/time.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

CBraid::CBraid()
{
	SetBnSize(500);
}

CBraid::CBraid(uInt32 bnSize)
{
	SetBnSize(bnSize);
}

CBraid::~CBraid()
{
	seed = 0;
}
 
uInt32 CBraid::GetBnSize()
{
	return bnSize;
}

void CBraid::SetBnSize(uInt32 bnSize)
{
	this->bnSize = bnSize;
}	

void CBraid::RandomLeftBraid(uInt32 braidResult[])
{
	uInt32 i;

	for(i = 0 ; i < bnSize ; i++)
	braidResult[i] = i;
	
	for(i = 0 ; i < bnSize / 2 ; i++)
	braidResult[i] = 0xFFFFFFFF;

	for(i = 0 ; i < bnSize / 2 ; i++)
	{
		uInt32 k = Random();

		while(braidResult[k % (bnSize / 2)] != 0xFFFFFFFF)
		k++;

		braidResult[k % (bnSize / 2)] = i;
	}
}

void CBraid::RandomRightBraid(uInt32 braidResult[])
{
	uInt32 i;

	for(i = 0 ; i < bnSize ; i++)
	braidResult[i] = i;
	
	for(i = bnSize / 2 ; i < bnSize ; i++)
	braidResult[i] = 0xFFFFFFFF;
	
	for(i = bnSize / 2 ; i < bnSize ; i++)
	{
		uInt32 k = Random();

		while(braidResult[(k % (bnSize / 2)) + (bnSize / 2)] != 0xFFFFFFFF)
		k++;

		braidResult[(k % (bnSize / 2)) + (bnSize / 2)] = i;
	}
}

void CBraid::RandomBraid(uInt32 braidResult[])
{
	uInt32 i;

	for(i = 0 ; i < bnSize ; i++)
	braidResult[i] = 0xFFFFFFFF;
	
	for(i = 0 ; i < bnSize ; i++)
	{
		uInt32 k = Random();

		while(braidResult[k % bnSize] != 0xFFFFFFFF)
		k++;

		braidResult[k % bnSize] = i;
	}
}

void CBraid::BraidToPassword(uInt32 braid[], uByte password[])
{
	uInt32 i, shift = 0;
	
	for(i = 0 ; i < 32 ; i++)
	password[i] = 0;

	for(shift = 0 ; shift < 32 ; shift += 8)
	{
		for(i = 0 ; i < bnSize ; i++)
		{
			uByte byte = (braid[i] >> shift) & 0xFF;

			if(byte)
			password[i % 32] ^= byte;
		}
	}
}

void CBraid::Reverse(uInt32 braid[], uInt32 braidResult[])
{
	for(uInt32 i = 0 ; i < bnSize ; i++)
	braidResult[braid[i]] = i;
}

void CBraid::Product(uInt32 braid1[], uInt32 braid2[], uInt32 braidResult[])
{
	for(uInt32 i = 0 ; i < bnSize ; i++)
	braidResult[i] = braid2[braid1[i]];
}

uInt32 CBraid::Random()
{
	timeval tv;
	uInt32 rnd;

	gettimeofday(&tv, NULL);
	seed ^= tv.tv_sec ^ tv.tv_usec;

	int fd_rnd = open("/dev/urandom", O_RDONLY);

	if(fd_rnd < 0)
	return seed;

	uInt32 cycle = seed & 0xF;

	for(uInt32 i = 0 ; i < cycle ; i++)
	{
		if(read(fd_rnd, &rnd, sizeof(uInt32)) < 0)
		{
			close(fd_rnd);
			return seed;
		}

		seed ^= rnd;
	}
	
	close(fd_rnd);
        
	return seed;
}
