/*
 *  KnotTTY is a remote secure shell using the new public key exchange protocol
 *  with the Braid Group Cryptosystems
 *
 *  Copyright (C) 2006 Adrien Pinet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 */

#include "CBuffer.h"

#include <stdlib.h>
#include <stdio.h>

CBuffer::CBuffer()
{
	buffer = NULL;
	bufferSize = 0;
	maxBufferSize = 0;
}

CBuffer::CBuffer(uInt32 maxBufferSize)
{
	this->buffer = NULL;
	this->bufferSize = 0;
	this->maxBufferSize = 0;

	SetBuffer(maxBufferSize);
}

CBuffer::CBuffer(uByte buffer[], uInt32 bufferSize)
{
	this->buffer = NULL;
	this->bufferSize = 0;
	this->maxBufferSize = 0;

	SetBuffer(buffer, bufferSize);
}
	
CBuffer::~CBuffer()
{
	Destroy();
}
	
int CBuffer::SetBuffer(uInt32 bufferSize)
{
	Destroy();

	buffer = new uByte[bufferSize];

	if(buffer == NULL)
	return 0;

	this->bufferSize = bufferSize;
	maxBufferSize = bufferSize;

	return 1;
}

int CBuffer::SetBuffer(uByte buffer[], uInt32 bufferSize)
{
	Destroy();

	this->buffer = new uByte[bufferSize];

	if(this->buffer == NULL)
	return 0;

	this->bufferSize = bufferSize;
	maxBufferSize = bufferSize;

	for(uInt32 i = 0 ; i < bufferSize ; i++)
	this->buffer[i] = buffer[i];

	return 1;
}

char* CBuffer::GetSignedBuffer()
{
	buffer[GetMaxBufferSize() - 1] = 0;
	return (char*) buffer;
}

uByte CBuffer::GetBuffer(uInt32 index)
{
	if(index >= GetMaxBufferSize())
	return 0;

	return buffer[index];
}

char CBuffer::GetSignedBuffer(uInt32 index)
{
	if(index >= GetMaxBufferSize())
	return 0;

	return (char) buffer[index];
}

uInt32 CBuffer::GetBufferSize()
{
	if(bufferSize > GetMaxBufferSize())
	bufferSize = GetMaxBufferSize();

	return bufferSize;
}




void CBuffer::Destroy()
{
	if(buffer)
	delete[] buffer;

	buffer = NULL;
	bufferSize = 0;
	maxBufferSize = 0;
}
	
