/*
 *  KnotTTY is a remote secure shell using the new public key exchange protocol
 *  with the Braid Group Cryptosystems
 *
 *  Copyright (C) 2006 Adrien Pinet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 */

#ifndef __CCHANNELMANAGER_H__
#define __CCHANNELMANAGER_H__

#include <pthread.h>

typedef unsigned char uByte;
typedef unsigned long uInt32;

template <class TConnection, class TEvent>
class CChannelManager
{
public:
	CChannelManager();
	virtual ~CChannelManager();

	int StartChannelManager(TConnection* pConnection, TEvent* pEvent, uInt32 maxChannel);
	int StopChannelManager();

	int Open(uInt32* pIdChannel);
	int Close(uInt32 idChannel);
	
	int Write(uInt32 idChannel, uByte data[], uInt32 dataSize);
	int Read(uInt32 idChannel, uByte data[], uInt32 maxDataSize, uInt32* pDataSize);

	uInt32 GetMaxChannel(){return maxChannel;};

private:
	static void* ThreadChannelManager(void* pvCChannelManager);
	
	int SendOpen(uInt32 idChannel);
	int SendClose(uInt32 idChannel);
	
private:
	class CChannel
	{
	public:
		CChannel();
		~CChannel();

		int Open();
		int Close();
		
		int IsOpen();
		int IsClose(){return !IsOpen();};

		int Write(uByte data[], uInt32 dataSize);
		int Read(uByte data[], uInt32 maxDataSize, uInt32* pDataSize);

	private:
		int IsDataFull();
		int IsDataEmpty(){return !IsDataFull();};

		int WriteData(uByte data[], uInt32 dataSize);
		int ReadData(uByte data[], uInt32 maxDataSize, uInt32* pDataSize);

		int FreeData();

	private:
		uByte* data;
		uInt32 dataSize;

		uByte isOpen;

		pthread_mutex_t mutexData;
		pthread_mutex_t mutexWait;
		pthread_cond_t condWait;
	};

	enum{OPEN = 0x00, DATA = 0x01, CLOSE = 0x02};
	
	TConnection* pConnection;
	TEvent* pEvent;
	CChannel* ChannelList;
	uInt32 maxChannel;

	pthread_t threadChannelManager;
};

#endif // __CCHANNELMANAGER_H__
