/*
 *  KnotTTY is a remote secure shell using the new public key exchange protocol
 *  with the Braid Group Cryptosystems
 *
 *  Copyright (C) 2006 Adrien Pinet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 */

#ifndef __CSECURESERVER_H__
#define __CSECURESERVER_H__

typedef unsigned char uByte;
typedef unsigned long uInt32;

#include "CAes.h"
#include <pthread.h> // for mutex

class CSecureServer
{
public:
	class CSecureConnection
	{
	public:
		enum {MAXSOCKETBUFFERSIZE = 0xFFFF, MAXSOCKETDATASIZE = 0xFFFD};

	public:
		CSecureConnection(int socketClient, char ip[]);
		~CSecureConnection();
	
		int SecureConnection();
		void Close();
		
		int SecureSend(uByte data[], uInt32 dataSize);
		int SecureReceive(uByte data[], uInt32 maxDataSize, uInt32* pDataSize);
	
		char* GetIp();
	
	private:
		int Send(uByte buffer[], uInt32 bufferSize);
		int Receive(uByte buffer[], uInt32 maxBufferSize, uInt32* pBufferSize);
	
		void LockToSend();
		void UnLockToSend();
		
		void LockToReceive();
		void UnLockToReceive();
	
		void ComputeCheckSum(uByte buffer[], uInt32 bufferSize, uByte checkSum[]);
		int VerifyCheckSum(uByte buffer[], uInt32 bufferSize, uByte checkSum[]);
	
	private:
		int socketClient;
		char ipClient[16];
	
		CAes AesToSend;
		CAes AesToReceive;
	
		int connectionSecured;
		
		pthread_mutex_t mutexToSend;
		pthread_mutex_t mutexToReceive;
	};

public:
	enum {NOERROR = 0x00, CLOSESERVERERROR = 0x01, CREATESERVERERROR = 0x02};

public:
	CSecureServer();
	~CSecureServer();

	int CreateServer(int portServer, int maxConnection);
	void CloseServer();

	CSecureConnection* WaitConnection();

	int GetMaxConnection()
	{return maxConnection;};

	uInt32 GetError();

private:
	uInt32 Error;
	int socketServer;
	int portServer;
	int maxConnection;
};

#endif // CSECURESERVER
