/*
 *  KnotTTY is a remote secure shell using the new public key exchange protocol
 *  with the Braid Group Cryptosystems
 *
 *  Copyright (C) 2006 Adrien Pinet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 */

#include "CSession.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <pthread.h>
#include <unistd.h>
#include <signal.h>

#include "CSessionShell.h"
#include "CSessionXShell.h"
#include "CSessionFTP.h"
#include "CBuffer.h"

int CSession::StartSession(CSecureServer::CSecureConnection* pSecureConnection)
{
	signal(SIGCHLD, SIG_IGN); // To avoid Zombie process after call '_exit'

	if(!pSecureConnection)
	return 0;

	if(fork() == 0)
	{
		CSession Session(pSecureConnection);
		Session.StopSession();
	}

	return 1;
}


CSession::CSession(CSecureServer::CSecureConnection* pSecureConnection)
{
	this->pSecureConnection = pSecureConnection;

	/******************************************************************************/
	/*                         Securisating connection                            */
	/******************************************************************************/
	printf("Incoming Connection : %s\n", pSecureConnection->GetIp());
	
	if(!pSecureConnection->SecureConnection())
	return;

	printf("Connection Secured\n");
	/******************************************************************************/
	/*                        Receiving session setting                           */
	/******************************************************************************/

	CBuffer UserName(100 + 1);
	CBuffer Password(100 + 1);
	CBuffer SessionMod(1);
	CBuffer Status(1);

	if(!pSecureConnection->SecureReceive(UserName.GetBuffer(), UserName.GetMaxBufferSize(), &UserName.bufferSize))
	return;

	if(!pSecureConnection->SecureReceive(Password.GetBuffer(), Password.GetMaxBufferSize(), &Password.bufferSize))
	return;

	if(!pSecureConnection->SecureReceive(SessionMod.GetBuffer(), SessionMod.GetMaxBufferSize(), &SessionMod.bufferSize))
	return;

	Authentication.SetAccount(UserName.GetSignedBuffer(), Password.GetSignedBuffer());

	if(!Authentication.IsAuthenticated())
	{
		Status.GetBuffer()[0] = NOAUTHENTICATED;
		pSecureConnection->SecureSend(Status.GetBuffer(), Status.GetMaxBufferSize());
		return;
	}

	if(!Authentication.ApplyProcessusId())
	{
		Status.GetBuffer()[0] = NOAUTHENTICATED;
		pSecureConnection->SecureSend(Status.GetBuffer(), Status.GetMaxBufferSize());
		return;
	}

	Status.GetBuffer()[0] = AUTHENTICATED;
	pSecureConnection->SecureSend(Status.GetBuffer(), Status.GetMaxBufferSize());
	
	if(SessionMod.GetBuffer(0) == SHELL)
	{
		CSessionShell SessionShell;
		SessionShell.Start(pSecureConnection, &Authentication);
	}

	if(SessionMod.GetBuffer(0) == XSHELL)
	{
		CSessionXShell SessionXShell;
		SessionXShell.Start(pSecureConnection, &Authentication);
	}

	if(SessionMod.GetBuffer(0) == FTP)
	{
		CSessionFTP SessionFTP;
		SessionFTP.Start(pSecureConnection, &Authentication);
	}
}


CSession::~CSession()
{
	StopSession();
}


void CSession::StopSession()
{
	if(pSecureConnection)
	delete pSecureConnection;
	
	_exit(0);
}


