/*
 *  KnotTTY is a remote secure shell using the new public key exchange protocol
 *  with the Braid Group Cryptosystems
 *
 *  Copyright (C) 2006 Adrien Pinet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 */

#include "CSessionFTP.h"

#include "CBuffer.h"
#include "CSHADigest.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <pthread.h>
#include <unistd.h>
#include <signal.h>

CSessionFTP::CSessionFTP()
{

}

CSessionFTP::~CSessionFTP()
{

}

int CSessionFTP::Start(CSecureServer::CSecureConnection* pSecureConnection, CAuthentication* pAuthentication)
{
	
	if(!pSecureConnection || !pAuthentication)
	return 0;

	this->pSecureConnection = pSecureConnection;
	this->pAuthentication = pAuthentication;

	CBuffer Request(1);

	while(pSecureConnection->SecureReceive(Request.GetBuffer(), Request.GetMaxBufferSize(), &Request.bufferSize))
	{
		switch(Request.GetBuffer(0))
		{
		case DOWNLOAD:
			if(!OnDownload())
			return 0;
			break;

		case UPLOAD:
			if(!OnUpload())
			return 0;
			break;

		case LS:
			if(!OnLS())
			return 0;
			break;
		case CD:
			if(!OnCD())
			return 0;
			break;

		case QUIT:
			return 1;

		default:
			return 0;
		}
	}

	return 0;
}

int CSessionFTP::OnDownload()
{
	CBuffer FilePathName(200);
	CBuffer Data(CSecureServer::CSecureConnection::MAXSOCKETDATASIZE);
	uInt32 fileSize;

	if(!pSecureConnection->SecureReceive(FilePathName.GetBuffer(), FilePathName.GetMaxBufferSize(), &FilePathName.bufferSize))
	return 0;
	
	if(!GetFileSize(FilePathName.GetSignedBuffer(), &fileSize))
	return 0;

	if(!pSecureConnection->SecureSend((uByte*) &fileSize, sizeof(fileSize)))
	return 0;

	FILE* pFile = fopen(FilePathName.GetSignedBuffer(), "r");

	if(pFile == NULL)
	return 0;

	uInt32 numBlock = fileSize / Data.GetMaxBufferSize();
	uInt32 lastBlockSize = fileSize % Data.GetMaxBufferSize();

	CSHADigest SHADigest;
	SHADigest.Start();

	for(uInt32 i = 0 ; i < numBlock ; i++)
	{
		fread(Data.GetBuffer(), sizeof(uByte), Data.GetMaxBufferSize(), pFile);
		SHADigest.ComputeInputBuffer(Data.GetBuffer(), Data.GetBufferSize());

		if(!pSecureConnection->SecureSend(Data.GetBuffer(), Data.GetMaxBufferSize()))
		{
			fclose(pFile);
			return 0;
		}
	}

	if(lastBlockSize)
	{
		fread(Data.GetBuffer(), sizeof(uByte), lastBlockSize, pFile);
		SHADigest.ComputeInputBuffer(Data.GetBuffer(), lastBlockSize);

		if(!pSecureConnection->SecureSend(Data.GetBuffer(), lastBlockSize))
		{
			fclose(pFile);
			return 0;
		}
	}

	SHADigest.Stop();
	fclose(pFile);

	if(!pSecureConnection->SecureSend(SHADigest.GetByteResult(), 20))
	return 0;

	return 1;
}

int CSessionFTP::OnUpload()
{
	CBuffer FilePathName(200);
	CBuffer Data(CSecureServer::CSecureConnection::MAXSOCKETDATASIZE);
	CBuffer FileSize(sizeof(uInt32));
	CBuffer SHADigestResult(20);
	uInt32 fileSize;

	if(!pSecureConnection->SecureReceive(FilePathName.GetBuffer(), FilePathName.GetMaxBufferSize(), &FilePathName.bufferSize))
	return 0;

	if(!pSecureConnection->SecureReceive((uByte*) &fileSize, FileSize.GetMaxBufferSize(), &FileSize.bufferSize))
	return 0;

	FILE* pFile = fopen(FilePathName.GetSignedBuffer(), "w");

	if(pFile == NULL)
	return 0;

	uInt32 numBlock = fileSize / Data.GetMaxBufferSize();
	uInt32 lastBlockSize = fileSize % Data.GetMaxBufferSize();

	CSHADigest SHADigest;
	SHADigest.Start();

	for(uInt32 i = 0 ; i < numBlock ; i++)
	{
		if(!pSecureConnection->SecureReceive(Data.GetBuffer(), Data.GetMaxBufferSize(), &Data.bufferSize))
		{
			fclose(pFile);
			return 0;
		}

		fwrite(Data.GetBuffer(), sizeof(uByte), Data.GetBufferSize(), pFile);
		SHADigest.ComputeInputBuffer(Data.GetBuffer(), Data.GetBufferSize());
	}

	if(lastBlockSize)
	{
		if(!pSecureConnection->SecureReceive(Data.GetBuffer(), Data.GetMaxBufferSize(), &Data.bufferSize))
		{
			fclose(pFile);
			return 0;
		}

		fwrite(Data.GetBuffer(), sizeof(uByte), Data.GetBufferSize(), pFile);
		SHADigest.ComputeInputBuffer(Data.GetBuffer(), Data.GetBufferSize());
	}

	SHADigest.Stop();
	fclose(pFile);

	if(!pSecureConnection->SecureReceive(SHADigestResult.GetBuffer(), SHADigestResult.GetMaxBufferSize(), &SHADigestResult.bufferSize))
	return 0;

	for(uInt32 i = 0 ; i < 20 ; i++)
	{
		if(SHADigest.GetByteResult()[i] != SHADigestResult.GetBuffer(i))
		return 0;
	}

	return 1;
}

int CSessionFTP::OnLS()
{

	return 1;
}

int CSessionFTP::OnCD()
{

	return 1;
}


int CSessionFTP::GetFileSize(char fileName[], uInt32* pFileSize)
{
	FILE* pFile = fopen(fileName, "r");

	if(pFile == NULL)
	return 0;
	
	if(fseek(pFile, 0, SEEK_END))
	{
		fclose(pFile);
		return 0;
	}

	*pFileSize = ftell(pFile);

	fclose(pFile);

	return 1;
}
