/*
 *  KnotTTY is a remote secure shell using the new public key exchange protocol
 *  with the Braid Group Cryptosystems
 *
 *  Copyright (C) 2006 Adrien Pinet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 */

#include "CSessionShell.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <pthread.h>
#include <unistd.h> // sleep
#include <signal.h>

#include "CBuffer.h"

CSessionShell::CSessionShell()
{

}

CSessionShell::~CSessionShell()
{
	Stop();
}

int CSessionShell::Start(CSecureServer::CSecureConnection* pSecureConnection, CAuthentication* pAuthentication)
{
	if(!pSecureConnection || !pAuthentication)
	return 0;

	this->pSecureConnection = pSecureConnection;
	this->pAuthentication = pAuthentication;

	if(!VirtualTerm.Create(pAuthentication, 0))
	return 0;

	StartThreadIOTerm();
	
	pthread_join(threadITerm, NULL);
	pthread_join(threadOTerm, NULL);

	return 1;
}

int CSessionShell::Stop()
{
	int ret = 1;
	
	if(pthread_cancel(threadITerm) != 0 && pthread_cancel(threadOTerm) != 0)
	ret = 0;

	VirtualTerm.Destroy();

	return ret;
}

int CSessionShell::StartThreadIOTerm()
{
	pthread_attr_init(&thread_attrITerm);
	pthread_attr_init(&thread_attrOTerm);

	if(pthread_create(&threadITerm, &thread_attrITerm, CSessionShell::ThreadITerm, this))
	return 0;

	if(pthread_create(&threadOTerm, &thread_attrOTerm, CSessionShell::ThreadOTerm, this))
	return 0;

	return 1;
}

void* CSessionShell::ThreadITerm(void* pvSessionShell)
{
	CSessionShell* pSessionShell = (CSessionShell*) pvSessionShell;

	CBuffer Data(CSecureServer::CSecureConnection::MAXSOCKETDATASIZE);
	
	while(pSessionShell->VirtualTerm.Read(Data.GetBuffer(), Data.GetMaxBufferSize(), &Data.bufferSize))
	{
		if(!pSessionShell->pSecureConnection->SecureSend(Data.GetBuffer(), Data.GetBufferSize()))
		{
			pthread_cancel(pSessionShell->threadOTerm);
			return NULL;
		}
	}

	pthread_cancel(pSessionShell->threadOTerm);
	return NULL;
}

void* CSessionShell::ThreadOTerm(void* pvSessionShell)
{
	CSessionShell* pSessionShell = (CSessionShell*) pvSessionShell;

	CBuffer Data(CSecureServer::CSecureConnection::MAXSOCKETDATASIZE);
	
	while(pSessionShell->pSecureConnection->SecureReceive(Data.GetBuffer(), Data.GetMaxBufferSize(), &Data.bufferSize))
	{
		if(!pSessionShell->VirtualTerm.Write(Data.GetBuffer(), Data.GetBufferSize()))
		{
			pthread_cancel(pSessionShell->threadITerm);
			return NULL;
		}
	}

	pthread_cancel(pSessionShell->threadITerm);
	return NULL;
}

