/*
 *  KnotTTY is a remote secure shell using the new public key exchange protocol
 *  with the Braid Group Cryptosystems
 *
 *  Copyright (C) 2006 Adrien Pinet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 */

#include "CSessionXShell.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <pthread.h>
#include <unistd.h>
#include <signal.h>

#include "CBuffer.h"

CSessionXShell::CSessionXShell()
{
	pSecureConnection = NULL;
	pAuthentication = NULL;

	for(uInt32 i = 0 ; i < 255 ; i++)
	xConnectionList[i].pXConnection = NULL;
}

CSessionXShell::~CSessionXShell()
{
	Stop();
}

int CSessionXShell::Start(CSecureServer::CSecureConnection* pSecureConnection, CAuthentication* pAuthentication)
{
	if(!pSecureConnection || !pAuthentication)
	return 0;

	this->pSecureConnection = pSecureConnection;
	this->pAuthentication = pAuthentication;

	int portXServer = 6020;

	while(XServer.CreateServer(portXServer, 200) == 0 && portXServer < 6120)
	portXServer++;
	
	if(portXServer == 6120)
	return 0;

	if(!VirtualTerm.Create(pAuthentication, portXServer))
	return 0;

	if(!ChannelManager.StartChannelManager(pSecureConnection, this, 255))
	return 0;

	if(!ChannelManager.Open(&idChannelTerm))
	return 0;

	StartThreadIOTerm();
	StartThreadXServer();

	pthread_join(threadITerm, NULL);
	pthread_join(threadOTerm, NULL);

	return 1;
}

int CSessionXShell::Stop()
{
	int ret = 1;

	pthread_cancel(threadITerm);
	pthread_cancel(threadOTerm);

	VirtualTerm.Destroy();

	for(uInt32 i = 0 ; i < 255 ; i++)
	{
		if(xConnectionList[i].pXConnection)
		{
			pthread_cancel(xConnectionList[i].threadIXConnection);
			pthread_cancel(xConnectionList[i].threadOXConnection);
			delete xConnectionList[i].pXConnection;
			xConnectionList[i].pXConnection = NULL;
		}
	}

	pthread_cancel(threadXServer);
	XServer.CloseServer();

	return ret;
}

int CSessionXShell::StartThreadIOTerm()
{
	pthread_attr_init(&thread_attrITerm);
	pthread_attr_init(&thread_attrOTerm);

	if(pthread_create(&threadITerm, &thread_attrITerm, CSessionXShell::ThreadITerm, this))
	return 0;

	if(pthread_create(&threadOTerm, &thread_attrOTerm, CSessionXShell::ThreadOTerm, this))
	return 0;

	return 1;
}

void* CSessionXShell::ThreadITerm(void* pvSessionXShell)
{
	CSessionXShell* pSessionShell = (CSessionXShell*) pvSessionXShell;

	CBuffer Data(CSecureServer::CSecureConnection::MAXSOCKETDATASIZE - 2);

	while(pSessionShell->VirtualTerm.Read(Data.GetBuffer(), Data.GetMaxBufferSize(), &Data.bufferSize))
	{
		if(!pSessionShell->ChannelManager.Write(0, Data.GetBuffer(), Data.GetBufferSize()))
		{
			pthread_cancel(pSessionShell->threadOTerm);
			pSessionShell->ChannelManager.Close(0);
			return NULL;
		}
	}

	pthread_cancel(pSessionShell->threadOTerm);
	pSessionShell->ChannelManager.Close(0);
	return NULL;
}

void* CSessionXShell::ThreadOTerm(void* pvSessionXShell)
{
	CSessionXShell* pSessionShell = (CSessionXShell*) pvSessionXShell;

	CBuffer Data(CSecureServer::CSecureConnection::MAXSOCKETDATASIZE - 2);

	while(pSessionShell->ChannelManager.Read(0, Data.GetBuffer(), Data.GetMaxBufferSize(), &Data.bufferSize))
	{
		if(!pSessionShell->VirtualTerm.Write(Data.GetBuffer(), Data.GetBufferSize()))
		{
			pthread_cancel(pSessionShell->threadITerm);
			pSessionShell->ChannelManager.Close(0);
			return NULL;
		}
	}

	pthread_cancel(pSessionShell->threadITerm);
	pSessionShell->ChannelManager.Close(0);
	return NULL;
}

int CSessionXShell::StartThreadXServer()
{
	pthread_attr_init(&thread_attrXServer);

	return pthread_create(&threadXServer, &thread_attrXServer, CSessionXShell::ThreadXServer, this) == 0;
}

void* CSessionXShell::ThreadXServer(void* pvSessionXShell)
{
	CSessionXShell* pSessionXShell = (CSessionXShell*) pvSessionXShell;
	CXServer::CXConnection* pXConnection;

	while((pXConnection = pSessionXShell->XServer.WaitConnection()) != NULL)
	{
		
		if(!pSessionXShell->StartThreadIOXConnection(pXConnection))
		delete pXConnection;
	}

	return NULL;
}

int CSessionXShell::StartThreadIOXConnection(CXServer::CXConnection* pXConnection)
{
	uInt32 idChannelXConnection;

	if(!ChannelManager.Open(&idChannelXConnection))
	return 0;

	xConnectionList[idChannelXConnection].idChannelXConnection = idChannelXConnection;
	xConnectionList[idChannelXConnection].pChannelManager = &ChannelManager;
	xConnectionList[idChannelXConnection].pXConnection = pXConnection;
	
	pthread_attr_init(&xConnectionList[idChannelXConnection].thread_attrIXConnection);
	pthread_attr_init(&xConnectionList[idChannelXConnection].thread_attrOXConnection);

	if(pthread_create(&xConnectionList[idChannelXConnection].threadIXConnection, &xConnectionList[idChannelXConnection].thread_attrIXConnection, CSessionXShell::ThreadIXConnection, &xConnectionList[idChannelXConnection]))
	return 0;

	if(pthread_create(&xConnectionList[idChannelXConnection].threadOXConnection, &xConnectionList[idChannelXConnection].thread_attrOXConnection, CSessionXShell::ThreadOXConnection, &xConnectionList[idChannelXConnection]))
	return 0;

	return 1;
}

void* CSessionXShell::ThreadIXConnection(void* pvThreadXConnectionData)
{
	SThreadXConnectionData* pThreadXConnectionData = (SThreadXConnectionData*) pvThreadXConnectionData;

	uInt32 idChannelXConnection = pThreadXConnectionData->idChannelXConnection;
	CXServer::CXConnection* pXConnection = pThreadXConnectionData->pXConnection;
	CChannelManager<CSecureServer::CSecureConnection, CSessionXShell>* pChannelManager = pThreadXConnectionData->pChannelManager;

	CBuffer Data(CSecureServer::CSecureConnection::MAXSOCKETDATASIZE - 2);

	while(pChannelManager->Read(idChannelXConnection, Data.GetBuffer(), Data.GetMaxBufferSize(), &Data.bufferSize))
	{
		if(!pXConnection->Send(Data.GetBuffer(), Data.GetBufferSize()))
		{
			pthread_cancel(pThreadXConnectionData->threadOXConnection);
			pChannelManager->Close(idChannelXConnection);
			delete pThreadXConnectionData->pXConnection;
			pThreadXConnectionData->pXConnection = NULL;
			return NULL;
		}
	}

	pthread_cancel(pThreadXConnectionData->threadOXConnection);
	pChannelManager->Close(idChannelXConnection);
	delete pThreadXConnectionData->pXConnection;
	pThreadXConnectionData->pXConnection = NULL;
	return NULL;
}

void* CSessionXShell::ThreadOXConnection(void* pvThreadXConnectionData)
{
	SThreadXConnectionData* pThreadXConnectionData = (SThreadXConnectionData*) pvThreadXConnectionData;
	
	uInt32 idChannelXConnection = pThreadXConnectionData->idChannelXConnection;
	CXServer::CXConnection* pXConnection = pThreadXConnectionData->pXConnection;
	CChannelManager<CSecureServer::CSecureConnection, CSessionXShell>* pChannelManager = pThreadXConnectionData->pChannelManager;

	CBuffer Data(CSecureServer::CSecureConnection::MAXSOCKETDATASIZE - 2);

	while(pXConnection->Receive(Data.GetBuffer(), Data.GetMaxBufferSize(), &Data.bufferSize))
	{
		if(!pChannelManager->Write(idChannelXConnection, Data.GetBuffer(), Data.GetBufferSize()))
		{
			pthread_cancel(pThreadXConnectionData->threadIXConnection);
			pChannelManager->Close(idChannelXConnection);
			delete pThreadXConnectionData->pXConnection;
			pThreadXConnectionData->pXConnection = NULL;
			return NULL;
		}
	}
	
	pthread_cancel(pThreadXConnectionData->threadIXConnection);
	pChannelManager->Close(idChannelXConnection);
	delete pThreadXConnectionData->pXConnection;
	pThreadXConnectionData->pXConnection = NULL;
	return NULL;
}

void CSessionXShell::OnOpen(uInt32 idChannel)
{

}

void CSessionXShell::OnClose(uInt32 idChannel)
{
	if(idChannel == idChannelTerm)
	{
		Stop();
		return;
	}

	if(xConnectionList[idChannel].pXConnection)
	{
		pthread_cancel(xConnectionList[idChannel].threadIXConnection);
		pthread_cancel(xConnectionList[idChannel].threadOXConnection);

		delete xConnectionList[idChannel].pXConnection;
		xConnectionList[idChannel].pXConnection = NULL;
	}
}
