/*
 *  KnotTTY is a remote secure shell using the new public key exchange protocol
 *  with the Braid Group Cryptosystems
 *
 *  Copyright (C) 2006 Adrien Pinet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 */

#ifndef __CSESSIONXSHELL_H__
#define __CSESSIONXSHELL_H__

#include "CSecureServer.h"
#include "CVirtualTerm.h"
#include "CAuthentication.h"
#include "CXServer.h"
#include "CChannelManager.h"
#include "CChannelManager.cpp"

#include <pthread.h>

class CSessionXShell
{
private:
	struct SThreadXConnectionData
	{
		CXServer::CXConnection* pXConnection;
		CChannelManager<CSecureServer::CSecureConnection, CSessionXShell>* pChannelManager;

		pthread_t threadIXConnection;
		pthread_t threadOXConnection;
		
		pthread_attr_t thread_attrIXConnection;
		pthread_attr_t thread_attrOXConnection;
		
		uInt32 idChannelXConnection;
	};

public:
	CSessionXShell();
	~CSessionXShell();

	int Start(CSecureServer::CSecureConnection* pSecureConnection, CAuthentication* pAuthentication);
	int Stop();

private:
	int StartThreadIOTerm();
	static void* ThreadITerm(void* pvSessionXShell);
	static void* ThreadOTerm(void* pvSessionXShell);

private:
	int StartThreadXServer();
	static void* ThreadXServer(void* pvSessionXShell);
	
	int StartThreadIOXConnection(CXServer::CXConnection* pXConnection);
	static void* ThreadIXConnection(void* pvThreadXConnectionData);
	static void* ThreadOXConnection(void* pvThreadXConnectionData);

public:
	void OnOpen(uInt32 idChannel);
	void OnClose(uInt32 idChannel);

private:
	CSecureServer::CSecureConnection* pSecureConnection;
	CAuthentication* pAuthentication;

	CVirtualTerm VirtualTerm;

	pthread_t threadITerm;
	pthread_t threadOTerm;
	pthread_t threadXServer;
	
	pthread_attr_t thread_attrITerm;
	pthread_attr_t thread_attrOTerm;
	pthread_attr_t thread_attrXServer;

	uInt32 idChannelTerm;

	CXServer XServer;
	CChannelManager<CSecureServer::CSecureConnection, CSessionXShell> ChannelManager;

	SThreadXConnectionData xConnectionList[255];
};

#endif // __CSESSIONXSHELL_H__
