/*
 *  KnotTTY is a remote secure shell using the new public key exchange protocol
 *  with the Braid Group Cryptosystems
 *
 *  Copyright (C) 2006 Adrien Pinet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 */

#include "CVirtualTerm.h"

#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <stropts.h>
#include <sys/ioctl.h>
#include <stdlib.h>
#include <string.h>

#include <stdio.h>


CVirtualTerm::CVirtualTerm()
{
	fdMaster = -1;
	shellPid = -1;
	slavePath = NULL;
}

CVirtualTerm::CVirtualTerm(CAuthentication* pAuthentication, int xServerPort)
{
	fdMaster = -1;
	shellPid = -1;
	slavePath = NULL;

	Create(pAuthentication, xServerPort);
}

CVirtualTerm::~CVirtualTerm()
{
	Destroy();
}

int CVirtualTerm::Create(CAuthentication* pAuthentication, int xServerPort)
{
	if(fdMaster != -1 || shellPid != -1)
	return 1;
	
	fdMaster = open("/dev/ptmx", O_RDWR | O_NOCTTY);

	if(fdMaster < 0)
	return 0;

	if(grantpt(fdMaster) < 0 || unlockpt(fdMaster) < 0)
	{
		close(fdMaster);
		return 0;
	}

	shellPid = fork();

	if(shellPid < 0)
	{
		close(fdMaster);
		return 0;
	}
	
	if(shellPid == 0)
	{
		setsid();

		slavePath = ptsname(fdMaster);
		int fdSlave = open(slavePath, O_RDWR | O_NOCTTY);

		dup2(fdSlave, 0);
		dup2(fdSlave, 1);
		dup2(fdSlave, 2);

		ioctl(fdSlave, I_PUSH, "ptem");
		ioctl(fdSlave, I_PUSH, "ldterm");

		close(fdSlave);

		char display[20];
		sprintf(display, "localhost:%d.0", xServerPort - 6000);

		setenv("HOME", pAuthentication->GetHomePath(), 1);
		setenv("DISPLAY", display, 1);
		chdir(pAuthentication->GetHomePath());

		execlp("/bin/bash", "-", "-login", NULL);
		return 0;
	}

	return 1;
}

void CVirtualTerm::Destroy()
{
	if(fdMaster == -1 || shellPid == -1)
	return;

	unlink(slavePath);
	close(fdMaster);
	killpg(shellPid, SIGKILL);

	fdMaster = -1;
	shellPid = -1;
	slavePath = NULL;
}

int CVirtualTerm::Write(uByte data[], uInt32 dataSize)
{
	if(fdMaster == -1 || shellPid == -1)
	return 0;

	uInt32 totalDataSize = 0;

	while(dataSize > totalDataSize)
	{
		int currentDataSize = write(fdMaster, data + totalDataSize, dataSize - totalDataSize);

		if(currentDataSize < 0)
		return 0;

		totalDataSize += currentDataSize;
	}

	return 1;
}

int CVirtualTerm::Read(uByte data[], uInt32 maxDataSize, uInt32* pDataSize)
{
	if(fdMaster == -1 || shellPid == -1)
	return 0;

	int dataSize = read(fdMaster, data, maxDataSize);

	if(dataSize < 0)
	return 0;

	*pDataSize = dataSize;

	return 1;
}
