/*
 *  KnotTTY is a remote secure shell using the new public key exchange protocol
 *  with the Braid Group Cryptosystems
 *
 *  Copyright (C) 2006 Adrien Pinet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 */

#include "CXServer.h"

#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <netdb.h>
#include <string.h>
#include <stdio.h>
#include <signal.h>

CXServer::CXConnection::CXConnection(int socketClient, char ipClient[])
{
	this->socketClient = socketClient;
	strcpy(this->ipClient, ipClient);

	pthread_mutex_init(&mutexToSend, NULL);
	pthread_mutex_init(&mutexToReceive, NULL);
}

CXServer::CXConnection::~CXConnection()
{
	Close();
}


void CXServer::CXConnection::Close()
{
	shutdown(socketClient, SHUT_RDWR);
	close(socketClient);
	
	pthread_mutex_unlock(&mutexToSend);
	pthread_mutex_unlock(&mutexToReceive);

	pthread_mutex_destroy(&mutexToSend);
	pthread_mutex_destroy(&mutexToReceive);
}

char* CXServer::CXConnection::GetIp()
{
	return ipClient;
}

int CXServer::CXConnection::Send(uByte data[], uInt32 dataSize)
{
	LockToSend();

	if(dataSize > MAXSOCKETDATASIZE)
	{
		UnLockToSend();
		return 0;
	}

	/* Sending trame */
	uInt32 dataSizeSend = 0;

	while(dataSizeSend < dataSize)
	{
		int currentDataSizeSend = write(socketClient, data + dataSizeSend, dataSize - dataSizeSend);
		
		if(currentDataSizeSend <= 0)
		{
			UnLockToSend();
			return 0;
		}

		dataSizeSend += currentDataSizeSend;
	}

	UnLockToSend();
	return 1;
}

int CXServer::CXConnection::Receive(uByte data[], uInt32 maxDataSize, uInt32* pDataSize)
{
	LockToReceive();

	if(maxDataSize > MAXSOCKETDATASIZE)
	maxDataSize = MAXSOCKETDATASIZE;

	int currentDataSizeReceive = read(socketClient, data, maxDataSize);

	if(currentDataSizeReceive <= 0)
	{
		UnLockToReceive();
		return 0;
	}

	*pDataSize = currentDataSizeReceive;

	UnLockToReceive();
	return 1;
}

void CXServer::CXConnection::LockToSend()
{
	pthread_mutex_lock(&mutexToSend);
}

void CXServer::CXConnection::UnLockToSend()
{
	pthread_mutex_unlock(&mutexToSend);
}

void CXServer::CXConnection::LockToReceive()
{
	pthread_mutex_lock(&mutexToReceive);
}

void CXServer::CXConnection::UnLockToReceive()
{
	pthread_mutex_unlock(&mutexToReceive);
}

CXServer::CXServer()
{
}

CXServer::~CXServer()
{
	CloseServer();
}

int CXServer::CreateServer(int portServer, int maxConnection)
{
	signal(SIGPIPE, SIG_IGN);
	
	this->maxConnection = maxConnection;
	this->portServer = portServer;
	socketServer = 0;
	
	addrinfo *res = NULL;
	addrinfo addrinfoServer;

	memset(&addrinfoServer, 0x00, sizeof(addrinfoServer));

	addrinfoServer.ai_family = AF_INET;
	addrinfoServer.ai_flags = AI_PASSIVE;
	addrinfoServer.ai_socktype = SOCK_STREAM;

	char sPortServer[6];
	sprintf(sPortServer, "%d", portServer);

	if (getaddrinfo(NULL, sPortServer, &addrinfoServer, &res) < 0)
	return 0;

	 /* Create the listen socket */
	 socketServer = socket(res->ai_family, res->ai_socktype, res->ai_protocol);

	if(socketServer < 0 || bind(socketServer, res->ai_addr, res->ai_addrlen) < 0 || listen(socketServer, maxConnection) < 0)
	{
		freeaddrinfo(res);
		return 0;
	}

	freeaddrinfo(res);
	return 1;
}

void CXServer::CloseServer()
{
	shutdown(socketServer, SHUT_RDWR);
	close(socketServer);
}

CXServer::CXConnection* CXServer::WaitConnection()
{
	int socketClient;
	char ipClient[16];
	
	sockaddr_in address;
	socklen_t adressSize = sizeof(sockaddr_in);

	socketClient = accept(socketServer, (sockaddr*)&address, &adressSize);

	if(socketClient < 0)
	return NULL;

	sprintf(ipClient, "%d.%d.%d.%d", (address.sin_addr.s_addr >> 0) & 0xFF,
						(address.sin_addr.s_addr >> 8) & 0xFF,
						(address.sin_addr.s_addr >> 16) & 0xFF,
						(address.sin_addr.s_addr >> 24) & 0xFF);

	return new CXConnection(socketClient, ipClient);
}
