/*
 *  KnotTTY is a remote secure shell using the new public key exchange protocol
 *  with the Braid Group Cryptosystems
 *
 *  Copyright (C) 2006 Adrien Pinet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 */

#ifndef __CXSERVER_H__
#define __CXSERVER_H__

typedef unsigned char uByte;
typedef unsigned long uInt32;

#include <pthread.h>

class CXServer
{
public:
	class CXConnection
	{
	public:
		enum {MAXSOCKETDATASIZE = 0xFFF0};

	public:
		CXConnection(int socketClient, char ip[]);
		~CXConnection();
	
		void Close();
		
		int Send(uByte data[], uInt32 dataSize);
		int Receive(uByte data[], uInt32 maxDataSize, uInt32* pDataSize);
	
		char* GetIp();
	
	private:	
		void LockToSend();
		void UnLockToSend();
		
		void LockToReceive();
		void UnLockToReceive();
	
	private:
		int socketClient;
		char ipClient[16];

		pthread_mutex_t mutexToSend;
		pthread_mutex_t mutexToReceive;
	};

public:
	CXServer();
	~CXServer();

	int CreateServer(int portServer, int maxConnection);
	void CloseServer();

	CXConnection* WaitConnection();

	int GetMaxConnection(){return maxConnection;};

private:
	uInt32 Error;
	int socketServer;
	int portServer;
	int maxConnection;
};

#endif // __CXSERVER_H__
